@@ -1,4 +1,4 @@
-// Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
+// Copyright 2018-2022 the Deno authors. All rights reserved. MIT license.
 use serde::Deserialize;
 
 use serde_v8::utils::{js_exec, v8_do};
@@ -271,3 +271,19 @@ detest!(de_bigint_i64, i64, "BigInt(-(2**59))", -(1 << 59));
 
 defail!(defail_struct, MathOp, "123", |e| e
   == Err(Error::ExpectedObject));
+
+#[derive(PartialEq, Debug, Deserialize)]
+pub struct SomeThing {
+  pub a: String,
+  #[serde(default)]
+  pub b: String,
+}
+detest!(
+  de_struct_defaults,
+  SomeThing,
+  "({ a: 'hello' })",
+  SomeThing {
+    a: "hello".into(),
+    b: "".into()
+  }
+);
